/*
 *  arch-tag: Implemetation files for podcast download manager
 *
 *  Copyright (C) 2005 Renato Araujo Oliveira Filho - INdT <renato.filho@indt.org.br>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <libgnomevfs/gnome-vfs-uri.h>
#include <glib/gstdio.h>
#include <gtk/gtk.h>
#include <string.h>
#define __USE_XOPEN
#include <time.h>

#include "rb-preferences.h"
#include "eel-gconf-extensions.h"
#include "rb-podcast-download.h"
#include "rb-file-helpers.h"
#include "rb-debug.h"
#include "rb-podcast-marshal.h"
#include "rhythmdb.h"
#include "rhythmdb-query-model.h"
#include "rb-podcast-parse.h"
#include "rb-metadata.h"


static void rb_podcast_download_class_init 		(RBPodcastDownloadClass *klass);

static void rb_podcast_download_init 			(RBPodcastDownload *dp);

static GObject *rb_podcast_download_constructor 	(GType type, guint n_construct_properties,
			   				 GObjectConstructParam *construct_properties);

static void rb_podcast_download_finalize 		(GObject *object);

static void rb_podcast_download_set_property 		(GObject *object,
                                   			 guint prop_id,
		                                	 const GValue *value,
		                                	 GParamSpec *pspec);

static void rb_podcast_download_get_property 		(GObject *object,
							 guint prop_id,
		                                	 GValue *value,
                		                	 GParamSpec *pspec);

static gpointer rb_podcast_download_copy_file 		(gpointer data);

static int rb_podcast_download_mkdir_with_parents 	(const gchar *pathname,
					  		 int mode);

static gint rb_podcast_download_date_sort_func 	  	(RhythmDBEntry *a, 
							 RhythmDBEntry *b);

static gboolean rb_podcast_download_start_thread 	(gpointer data);

static gboolean rb_podcast_download_sync_head_cb 	(gpointer data);

static gboolean rb_podcast_download_head_query_cb 	(GtkTreeModel *query_model,
						   	 GtkTreePath *path, 
							 GtkTreeIter *iter,
						   	 RhythmDB *data);

static gboolean rb_podcast_download_save_metadata	(RhythmDB *db, 
						  	 RhythmDBEntry *entry, 
						  	 const char* uri);

static void rb_podcast_download_db_entry_added_cb 	(RBPodcastDownload *pd, 
							 RhythmDBEntry *entry);


#define CONF_STATE_PODCAST_DOWNLOAD_HOUR        CONF_PREFIX "/state/podcast/download_hour"
#define CONF_STATE_PODCAST_DOWNLOAD_MINUTE      CONF_PREFIX "/state/podcast/download_minute"
#define CONF_STATE_PODCAST_DOWNLOAD_NEXT_TIME   CONF_PREFIX "/state/podcast/download_next_time"

struct RBPodcastDownloadPrivate
{
	RhythmDB *db;
	GList *download_list;
	GThread *copy_thread;
	GThread *sync_thread;
	gboolean cancel_thread;
	guint source_sync;
};

enum 
{
	PROP_0,
	PROP_DB
};

enum
{
	STATUS_CHANGED,
	LAST_SIGNAL
};

static guint rb_podcast_download_signals[LAST_SIGNAL] = { 0 };
static GStaticMutex entry_add_mutex = G_STATIC_MUTEX_INIT;

#define BUFFER_SIZE 256


// Prefix
#define CONF_STATE_PODCAST_SAVE_DIR	CONF_PREFIX "/state/podcast/save_dir"

static GObjectClass *parent_class = NULL;

GType
rb_podcast_download_get_type (void)
{
	static GType rb_podcast_download_type = 0;

	if (rb_podcast_download_type == 0)
	{
		static const GTypeInfo our_info =
		{
			sizeof (RBPodcastDownloadClass),
			NULL,
			NULL,
			(GClassInitFunc) rb_podcast_download_class_init,
			NULL,
			NULL,
			sizeof (RBPodcastDownload),
			0,
			(GInstanceInitFunc) rb_podcast_download_init
		};

		rb_podcast_download_type = g_type_register_static (G_TYPE_OBJECT,
							     "RBPodcastDownload",
							     &our_info, 0);
	}

	return rb_podcast_download_type;

}



static void
rb_podcast_download_class_init (RBPodcastDownloadClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);
	
	parent_class = g_type_class_peek_parent (klass);

	object_class->constructor = rb_podcast_download_constructor;	
	object_class->finalize = rb_podcast_download_finalize;

	object_class->set_property = rb_podcast_download_set_property;
        object_class->get_property = rb_podcast_download_get_property;

	 g_object_class_install_property (object_class,
                                          PROP_DB,
                                          g_param_spec_pointer ("db",
							       	"db",
							       	"database",
								G_PARAM_READWRITE));

	rb_podcast_download_signals[STATUS_CHANGED] = 
	       g_signal_new ("status_changed",
		       		G_OBJECT_CLASS_TYPE (object_class),
		 		GTK_RUN_LAST,
				G_STRUCT_OFFSET (RBPodcastDownloadClass, status_changed),
				NULL, NULL,
				rb_podcast_marshal_VOID__POINTER_DOUBLE,
				G_TYPE_NONE,
				2,
				G_TYPE_POINTER,
				G_TYPE_DOUBLE);
					
}	

static void
rb_podcast_download_init (RBPodcastDownload *pd)
{
	pd->priv = g_new0 (RBPodcastDownloadPrivate, 1);
	pd->priv->source_sync = 0;
	pd->priv->copy_thread = 0;
	eel_gconf_monitor_add (CONF_PREFIX);
}

static GObject *
rb_podcast_download_constructor (GType type, guint n_construct_properties,
			   GObjectConstructParam *construct_properties)
{
	RBPodcastDownload *pd;

	pd = RB_PODCAST_DOWNLOAD (G_OBJECT_CLASS (parent_class)
					->constructor (type, n_construct_properties, construct_properties));
	
	return G_OBJECT (pd);

}

static void
rb_podcast_download_finalize (GObject *object)
{
	RBPodcastDownload *pd;
	g_return_if_fail (object != NULL);
        g_return_if_fail (RB_IS_PODCAST_DOWNLOAD (object));

	pd = RB_PODCAST_DOWNLOAD(object);
	
	g_return_if_fail (pd->priv != NULL);

	eel_gconf_monitor_remove (CONF_PREFIX);
	
	if (pd->priv->download_list)
		g_list_free (pd->priv->download_list);

	if (pd->priv->source_sync) {
		g_source_remove (pd->priv->source_sync);
		pd->priv->source_sync = 0;
	}

	if (pd->priv->copy_thread != 0) {
		rb_debug("Waiting copy thread");
		pd->priv->cancel_thread = TRUE;
		g_thread_join (pd->priv->copy_thread);
	}

	g_free(pd->priv);
	
	G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void 
rb_podcast_download_set_property (GObject *object,
                           		guint prop_id,
	                                const GValue *value,
	                                GParamSpec *pspec)
{
	RBPodcastDownload *pd = RB_PODCAST_DOWNLOAD (object);

	switch (prop_id)
	{
	case PROP_DB:
		pd->priv->db = g_value_get_pointer (value);
	        g_signal_connect_object (G_OBJECT (pd->priv->db),
	                                 "entry-added",
	                                 G_CALLBACK (rb_podcast_download_db_entry_added_cb),
	                                 pd, G_CONNECT_SWAPPED);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
	}
}

static void 
rb_podcast_download_get_property (GObject *object,
					guint prop_id,
		                        GValue *value,
                		        GParamSpec *pspec)
{
	RBPodcastDownload *pd = RB_PODCAST_DOWNLOAD (object);

	switch (prop_id)
	{
	case PROP_DB:
		g_value_set_pointer(value, pd->priv->db);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
	}

	
}

RBPodcastDownload *
rb_podcast_download_new (RhythmDB *db)
{
	RBPodcastDownload *pd;

	pd = g_object_new (RB_TYPE_PODCAST_DOWNLOAD, "db", db, NULL);
	return pd; 
}


void
rb_podcast_download_add_entry (RBPodcastDownload *pd, RhythmDBEntry *entry)
{
	double status;
	if (entry == NULL)
		return;


	
	status = entry->status; 
	if (status != 100) {
		
		if (status != 0) {
			GValue status_val = { 0, };
			g_value_init (&status_val, G_TYPE_DOUBLE);
			g_value_set_double (&status_val, 0);
			rhythmdb_entry_set (pd->priv->db, entry, RHYTHMDB_PROP_STATUS, &status_val);
			g_value_unset (&status_val);
		}
	
		rb_debug("Try insert entry for download.");

        	g_static_mutex_lock (&entry_add_mutex);
		pd->priv->download_list =  g_list_insert_sorted (pd->priv->download_list, 
						 		 (void *) entry, 
								 (GCompareFunc) rb_podcast_download_date_sort_func);	

		g_static_mutex_unlock (&entry_add_mutex);

		rb_debug("entry inserted.");
		gtk_idle_add(rb_podcast_download_start_thread, (gpointer ) pd);
	}
	
	
}


void
rb_podcast_download_start_sync(RBPodcastDownload *pd)
{
	gint next_time =  eel_gconf_get_integer(CONF_STATE_PODCAST_DOWNLOAD_NEXT_TIME);


	if (pd->priv->source_sync != 0) {
		g_source_remove (pd->priv->source_sync);
		pd->priv->source_sync = 0;
	}

		
	if (next_time > 0) {
		pd->priv->source_sync = g_timeout_add (next_time * 1000, (GSourceFunc ) rb_podcast_download_sync_head_cb, pd);
	}

}

static gboolean
rb_podcast_download_sync_head_cb (gpointer data)
{
	RBPodcastDownload *pd = (RBPodcastDownload *) data;
	
	GtkTreeModel* query_model = GTK_TREE_MODEL (rhythmdb_query_model_new_empty(pd->priv->db));	


	rhythmdb_do_full_query (pd->priv->db, query_model,
                                RHYTHMDB_QUERY_PROP_EQUALS,
                                RHYTHMDB_PROP_TYPE,
                                RHYTHMDB_ENTRY_TYPE_PODCAST_HEAD,
                                RHYTHMDB_QUERY_END);

 	gtk_tree_model_foreach (query_model,
		                (GtkTreeModelForeachFunc)rb_podcast_download_head_query_cb,
                                pd->priv->db);

	rb_podcast_download_start_sync (pd); 

	return FALSE;
 
}
	
static gboolean
rb_podcast_download_head_query_cb (GtkTreeModel *query_model,
					GtkTreePath *path, GtkTreeIter *iter,
					RhythmDB *data)
{
        const char* uri;
        RhythmDBEntry* entry;

        gtk_tree_model_get (query_model, iter, 0, &entry, -1);
        uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);

	rb_podcast_download_add_url(data, uri);
	
        return FALSE;
}
	
static gboolean
rb_podcast_download_start_thread (gpointer data)
{
	rb_debug("Stating thread for download podcast files.");
	RBPodcastDownload *pd = (RBPodcastDownload *) data;
	if (pd->priv->copy_thread == 0)
		pd->priv->copy_thread = g_thread_create(rb_podcast_download_copy_file, (gpointer ) pd, TRUE, NULL);

	return FALSE;

}

static gpointer 
rb_podcast_download_copy_file (gpointer data)
{
	guint buffer[BUFFER_SIZE];	
	
	GnomeVFSResult ret = 0;
    	GnomeVFSHandle *read_handle = NULL;
    	GnomeVFSHandle *write_handle = NULL;
	GnomeVFSFileInfo *info_uri;
	
	GValue status_val = { 0, };
	GValue location_val = { 0, };
	
    	GnomeVFSFileSize bytes_read;
    	GnomeVFSFileSize bytes_write;
    	GnomeVFSFileSize total_size = 0;
    	GnomeVFSFileSize read_size = 0;
	guint step;
	guint step_old;
	gboolean abandon = FALSE;

	const char *short_name;
	const char *local_file_name;
	const char *dir_name;
	const char *conf_dir_name;
	GnomeVFSURI *remote_uri = NULL;
	GnomeVFSURI *local_uri = NULL;
	
	
	RBPodcastDownload *pd = (RBPodcastDownload *) data;

	rb_debug("Starting loop for copy file..");
	

	g_value_init (&location_val, G_TYPE_STRING);

	conf_dir_name = eel_gconf_get_string(CONF_STATE_PODCAST_SAVE_DIR);
	
	while ((g_list_length(pd->priv->download_list) > 0) && !abandon) {
		const char *location; 
		const char *album_name; 
		
		g_static_mutex_lock (&entry_add_mutex);
		RhythmDBEntry *entry = (RhythmDBEntry *) g_list_first(pd->priv->download_list)->data;
		g_static_mutex_unlock (&entry_add_mutex);

		rb_debug("Downloading file..");
		if (entry == NULL) { 
			rb_debug("File is null..");
			goto next_step;
		}
		
		location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
		album_name = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM);
		
	
		if (remote_uri) 
			gnome_vfs_uri_unref(remote_uri);
		
		remote_uri = gnome_vfs_uri_new(location);
		if (!remote_uri) 
			goto next_step;
		
		if (gnome_vfs_uri_is_local(remote_uri))	
			goto next_step;

		short_name = gnome_vfs_uri_extract_short_name(remote_uri);
		
		info_uri = gnome_vfs_file_info_new();
		ret = gnome_vfs_get_file_info_uri(remote_uri, info_uri, GNOME_VFS_FILE_INFO_DEFAULT);
		
		if (ret == GNOME_VFS_OK) {
			total_size = info_uri->size;
		}
		
		//verify if dir existis
		dir_name = g_build_filename (g_get_home_dir (),
						(conf_dir_name == NULL ? "My Podcast" : conf_dir_name),
						album_name,
						NULL);
		
		 if (!g_file_test (dir_name, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
			if (rb_podcast_download_mkdir_with_parents (dir_name, 0750) != 0) {
				rb_debug("Error on create local dirs.");
				abandon = TRUE;
				goto next_step;
			}
			
				
		 }
	
	 		 
		local_file_name = g_build_filename (dir_name,
							short_name,
							NULL);	

	
		ret = gnome_vfs_open_uri (&read_handle, remote_uri, GNOME_VFS_OPEN_READ | GNOME_VFS_OPEN_RANDOM);
		if (ret != GNOME_VFS_OK) { 
			rb_debug("Error on open remote file.");
			goto next_step;
		}
		
		if (local_uri)
			gnome_vfs_uri_unref(local_uri);
		
		local_uri = gnome_vfs_uri_new (local_file_name);
		if (!local_uri) {
			goto next_step;
		}

		//TODO: suport resume file
		if (g_file_test (local_file_name, G_FILE_TEST_EXISTS)) {
			rb_debug("Removing existent file.");
			g_remove(local_file_name);
		}
		
		ret = gnome_vfs_create_uri (&write_handle, local_uri, GNOME_VFS_OPEN_WRITE, FALSE, 0777);
		if (ret != GNOME_VFS_OK) {
				rb_debug("Error on create local file [%s].", local_file_name);
				goto next_step;
		} 
		
	
		step = 0;
		step_old = 0;

		ret = gnome_vfs_read(read_handle, &buffer, BUFFER_SIZE, &bytes_read);
		read_size = 0;	
		while(ret == GNOME_VFS_OK) {
			gnome_vfs_write(write_handle, buffer, bytes_read, &bytes_write);
			read_size += bytes_read;
			step =  (gint) 100*read_size/total_size;
			
			if (step != step_old) {
				if (pd->priv->cancel_thread) {
					break;
				}
	
				rb_debug("Emiting status_change signal.");
				entry->status = step;
				
				GDK_THREADS_ENTER ();
				
				g_signal_emit (pd, rb_podcast_download_signals[STATUS_CHANGED],
						0, entry, step);
				GDK_THREADS_LEAVE ();
				
				step_old = step;

			}
			step++;
			ret = gnome_vfs_read(read_handle, &buffer, BUFFER_SIZE, &bytes_read);
		}
		if (pd->priv->cancel_thread)
			break;

next_step:
		if (ret == GNOME_VFS_ERROR_EOF) {
			g_value_set_string (&location_val, gnome_vfs_uri_to_string(local_uri, GNOME_VFS_URI_HIDE_NONE));
			
			g_value_init (&status_val, G_TYPE_UINT64);
			g_value_set_uint64 (&status_val, read_size);
			rhythmdb_entry_set (pd->priv->db, entry, RHYTHMDB_PROP_FILE_SIZE, &status_val);
			g_value_unset (&status_val);
		
			rb_podcast_download_save_metadata(pd->priv->db, entry, g_value_get_string (&location_val));
			
			g_value_init (&status_val, G_TYPE_DOUBLE);
			g_value_set_double (&status_val, 100);
			rhythmdb_entry_set (pd->priv->db, entry, RHYTHMDB_PROP_STATUS, &status_val);
			rhythmdb_entry_set (pd->priv->db, entry, RHYTHMDB_PROP_MOUNTPOINT, &location_val);
			g_value_unset (&status_val);
			rhythmdb_commit(pd->priv->db);
		}
		else {
			entry->status = -1;
		}
	
		if (read_handle != NULL) {
			gnome_vfs_close(read_handle);
			read_handle = NULL;
		}

		if (write_handle != NULL) {
			gnome_vfs_close(write_handle);
			write_handle = NULL;
		}

		g_static_mutex_lock (&entry_add_mutex);
		pd->priv->download_list = g_list_remove(pd->priv->download_list, (gconstpointer ) entry);
		g_static_mutex_unlock (&entry_add_mutex);
	}
	

	
	if (read_handle != NULL)
		gnome_vfs_close(read_handle);

	if (write_handle != NULL)
		gnome_vfs_close(write_handle);
	
	if (local_uri)
		gnome_vfs_uri_unref(local_uri);

	if (remote_uri)
		gnome_vfs_uri_unref(remote_uri);
	

	rb_debug("Copy thread exiting..");
	
	return NULL;
}

static int
rb_podcast_download_mkdir_with_parents (const gchar *pathname,
		   			int          mode)
{
  gchar *fn, *p;

  if (pathname == NULL || *pathname == '\0')
    {
      return -1;
    }

  fn = g_strdup (pathname);

  if (g_path_is_absolute (fn))
    p = (gchar *) g_path_skip_root (fn);
  else
    p = fn;

  do
    {
      while (*p && !G_IS_DIR_SEPARATOR (*p))
	p++;
      
      if (!*p)
	p = NULL;
      else
	*p = '\0';
      
      if (!g_file_test (fn, G_FILE_TEST_EXISTS))
	{
	  if (g_mkdir (fn, mode) == -1)
	    {
	      g_free (fn);
	      return -1;
	    }
	}
      else if (!g_file_test (fn, G_FILE_TEST_IS_DIR))
	{
	  g_free (fn);
	  return -1;
	}
      if (p)
	{
	  *p++ = G_DIR_SEPARATOR;
	  while (*p && G_IS_DIR_SEPARATOR (*p))
	    p++;
	}
    }
  while (p);

  g_free (fn);

  return 0;
}

static gint
rb_podcast_download_date_sort_func (RhythmDBEntry *a, RhythmDBEntry *b)
{
	const char *a_val;
	const char *b_val;	
	struct tm a_tm;
	struct tm b_tm;
	time_t a_time;
	time_t b_time;

	gint ret = 0;


	a_val =  rb_refstring_get (a->album); //rhythmdb_entry_get_string (a, RHYTHMDB_PROP_ALBUM);
	b_val =  rb_refstring_get (b->album); //rhythmdb_entry_get_string (b, RHYTHMDB_PROP_ALBUM);

	ret = strcmp(a_val, b_val);
	if (ret != 0) {
		return 0;	
	}
	
	
	a_val = rb_refstring_get (a->date_str); // rhythmdb_entry_get_string (a, RHYTHMDB_PROP_DATE_STR);
	strptime(a_val, "%d/%m/%Y %H:%M", &a_tm);
	a_tm.tm_sec = 0;
	a_time = mktime(&a_tm);
	
	b_val = rb_refstring_get (b->date_str); // rhythmdb_entry_get_string (b, RHYTHMDB_PROP_DATE_STR);
	strptime(b_val, "%d/%m/%Y %H:%M", &b_tm);
	b_tm.tm_sec = 0;
	b_time = mktime(&b_tm);

	ret = difftime(a_time,b_time); 
	ret = (ret == 0 ? 0 : (ret > 0 ? 1 : -1));
	return ret;
}

gboolean
rb_podcast_download_add_url (RhythmDB *db, const char* url)
{
	GValue description_val = { 0, };
	GValue title_val = { 0, };
	GValue subtitle_val = { 0, };
	GValue summary_val = { 0, };
	GValue lang_val = { 0, };
	GValue copyright_val = { 0, };
	GValue image_val = { 0, };
	GValue author_val = { 0, };
	GValue date_val = { 0, };

	RhythmDBEntry *entry;

	GList *lst_songs;
	
	RBPodcastChannel *data = g_new0(RBPodcastChannel, 1);
	
	rb_debug("Inserting podcast head - %s\n", url);
	
	rb_podcast_parse_load_feed(data, url);

	if (data->title	== NULL) {
		g_list_free(data->lst_itens);
		g_free(data);
		return FALSE;
	}

	//processing podcast head
	entry = rhythmdb_entry_lookup_by_location (db, url);
	if (entry)
		goto load_posts;
	
	entry = rhythmdb_entry_new (db,
				    RHYTHMDB_ENTRY_TYPE_PODCAST_HEAD,
			    	    url);


	rb_debug("New entry create\n");
	
	g_value_init (&title_val, G_TYPE_STRING);
	g_value_set_string(&title_val, data->title);
	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_TITLE, &title_val);
	g_value_unset (&title_val);

	rb_debug("Filed title fill\n");
	if (!data->author)
		data->author = strdup("<unknowm>");
	
	g_value_init (&author_val, G_TYPE_STRING);
	g_value_set_string(&author_val, data->author);
	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_ARTIST, &author_val);
	g_value_unset (&author_val);
	
	rb_debug("Field author fill\n");
	
	if (data->subtitle) {
		g_value_init (&subtitle_val, G_TYPE_STRING);
		g_value_set_string(&subtitle_val, data->subtitle);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_SUBTITLE, &subtitle_val);
		g_value_unset (&subtitle_val);
	}
	
	if (data->description) {
		g_value_init (&description_val, G_TYPE_STRING);
		g_value_set_string(&description_val, data->description);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_DESCRIPTION, &description_val);
		g_value_unset (&description_val);
	}
	
	if (data->summary) {
		g_value_init (&summary_val, G_TYPE_STRING);
		g_value_set_string(&summary_val, data->summary);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_SUMMARY, &summary_val);
		g_value_unset (&summary_val);
	}

	if (data->lang) {
		g_value_init (&lang_val, G_TYPE_STRING);
		g_value_set_string(&lang_val, data->lang);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_LANG, &lang_val);
		g_value_unset (&lang_val);
	}

	if (data->copyright) { 
		g_value_init (&copyright_val, G_TYPE_STRING);
		g_value_set_string(&copyright_val, data->copyright);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_COPYRIGHT, &copyright_val);
		g_value_unset (&copyright_val);
	}

	if (data->pub_date) {
		g_value_init (&date_val, G_TYPE_STRING);
		g_value_set_string(&date_val, data->pub_date);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_DATE_STR, &date_val);
		g_value_unset (&date_val);
	}		

	if (data->img) {
		g_value_init (&image_val, G_TYPE_STRING);
		g_value_set_string(&image_val, data->img);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_IMAGE, &image_val);
		g_value_unset (&image_val);
	}



	rhythmdb_commit (db);
	
	rb_debug("Podcast head Inserted");
	
load_posts:
	
	lst_songs = data->lst_itens;
	while (lst_songs) {
		RBPodcastItem *item = (RBPodcastItem *) lst_songs->data;
		if (item) {
			rb_podcast_download_add_post(db, 
						data->title,
						item->title, 
						data->author,
						item->url,
						item->pub_date);
		}
		lst_songs = lst_songs->next;
	}

	
	
	g_list_free(data->lst_itens);
	return TRUE;
}

void
rb_podcast_download_add_post (RhythmDB *db,
			      const char *name,
			      const char *title,
			      const char *generator,
			      const char *uri,
			      const char *date)
{
	//GThread* copy_thread = NULL;
	GValue title_val = { 0, };
	GValue artist_val = { 0, };
	GValue album_val = { 0, };
	GValue mountpoit_val = { 0, };
	GValue status_val = { 0, };
	GValue date_val = { 0, };
	GValue genre_val = { 0, };
	
	RhythmDBEntry *entry;

	if (uri && name && title && date && g_utf8_validate(uri, -1, NULL)) {
		entry = rhythmdb_entry_lookup_by_location (db, uri);

		if (entry)
			return;

		g_value_init (&album_val, G_TYPE_STRING);
		g_value_set_string (&album_val, name);
		
		g_value_init (&mountpoit_val, G_TYPE_STRING);
		g_value_set_string (&mountpoit_val, uri);
	
		g_value_init (&status_val, G_TYPE_DOUBLE);
		g_value_set_double (&status_val, 0);

		g_value_init (&genre_val, G_TYPE_STRING);
		g_value_set_string (&genre_val, "Podcast");

		g_value_init (&date_val, G_TYPE_STRING);
		g_value_set_string (&date_val, date);
		
		g_value_init (&title_val, G_TYPE_STRING);
		g_value_set_string (&title_val, title);
	
		g_value_init (&artist_val, G_TYPE_STRING);
		if (generator) 
			g_value_set_string (&artist_val, generator);
		else
			g_value_set_string (&artist_val, "");


		
		entry = rhythmdb_entry_new (db,
					    RHYTHMDB_ENTRY_TYPE_PODCAST_POST,
				    	    uri);

		rb_debug("entry created");
			
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_GENRE, &genre_val);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_DATE_STR, &date_val);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_TITLE, &title_val);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_ARTIST, &artist_val);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_ALBUM, &album_val);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_MOUNTPOINT, &mountpoit_val);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_STATUS, &status_val);
			
		rhythmdb_commit (db);

		rb_debug("podcast entry inserted\n");


		g_value_unset (&date_val);
		g_value_unset (&title_val);
		g_value_unset (&album_val);
		g_value_unset (&mountpoit_val);
		g_value_unset (&status_val);
		g_value_unset (&artist_val);
	}

}

static gboolean
rb_podcast_download_save_metadata(RhythmDB *db, RhythmDBEntry *entry, const char* uri)
{
	RBMetaData *md = rb_metadata_new();
	GError *error = NULL;
	GValue val_duration = { 0, };
	GValue val_bitrate = { 0, };
	
        rb_metadata_load (md, uri, &error);	

	if (error != NULL) {
		g_object_unref (md);
		return FALSE;
	}

	if (rb_metadata_get (md,
				RB_METADATA_FIELD_DURATION,
				&val_duration)) {
		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_DURATION, &val_duration);
		g_value_unset (&val_duration);
	}

	if (rb_metadata_get (md, 
				RB_METADATA_FIELD_BITRATE,
				&val_bitrate)) {
		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_BITRATE, &val_bitrate);
		g_value_unset (&val_bitrate);
	}
	g_object_unref (md);
	return TRUE;
}

static void
rb_podcast_download_db_entry_added_cb (RBPodcastDownload *pd, RhythmDBEntry *entry)
{
	if (entry->type != RHYTHMDB_ENTRY_TYPE_PODCAST_POST)
		return;
	
        rb_podcast_download_add_entry(pd, entry);
}

