/* 
 *  arch-tag: Implementation of Internet Podcast source object
 *
 *  Copyright (C) 2005 Renato Araujo Oliveira Filho <renato.filho@indt.org.br>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <config.h>
#include <gtk/gtk.h>
#include <gtk/gtkspinbutton.h>
#include <gtk/gtkbutton.h>
#include <glib/gfileutils.h>
#include <glade/glade.h>
#include <libgnome/gnome-i18n.h>
#include <libxml/tree.h>
#include <string.h>

#include "rb-podcast-source.h"

#include "rhythmdb-query-model.h"
#include "rb-glade-helpers.h"
#include "rb-stock-icons.h"
#include "rb-entry-view.h"
#include "rb-property-view.h"
#include "rb-util.h"
#include "rb-file-helpers.h"
#include "totem-pl-parser.h"
#include "rb-preferences.h"
#include "rb-dialog.h"
#include "rb-new-podcast-dialog.h"
#include "rb-podcast-properties-dialog.h"
//#include "rb-volume.h"
#include "rb-debug.h"
#include "rb-statusbar.h"
#include "eel-gconf-extensions.h"
#include "rb-podcast-download.h"

typedef enum
{
	RB_PODCAST_QUERY_TYPE_ALL,
	RB_PODCAST_QUERY_TYPE_ALBUM,
	RB_PODCAST_QUERY_TYPE_SEARCH,
} RBPodcastQueryType;

static void rb_podcast_source_class_init 		(RBPodcastSourceClass *klass);

static void rb_podcast_source_init 			(RBPodcastSource *source);

static GObject *rb_podcast_source_constructor 		(GType type, 
							 guint n_construct_properties,
					      		 GObjectConstructParam *construct_properties);

static void rb_podcast_source_dispose 			(GObject *object);

static void rb_podcast_source_finalize 			(GObject *object);

static void rb_podcast_source_set_property 		(GObject *object,
			                  		 guint prop_id,
			                  		 const GValue *value,
			                  		 GParamSpec *pspec);

static void rb_podcast_source_get_property 		(GObject *object,
			                  		 guint prop_id,
			                  		 GValue *value,
			                  		 GParamSpec *pspec);

static void rb_podcast_source_songs_show_popup_cb 	(RBEntryView *view,
						  	 RBPodcastSource *source);

static void paned_size_allocate_cb 			(GtkWidget *widget,
				    			 GtkAllocation *allocation,
		                    			 RBPodcastSource *source);
				    
static void rb_podcast_source_state_pref_changed 	(GConfClient *client,
						 	 guint cnxn_id,
						 	 GConfEntry *entry,
					 		 RBPodcastSource *source);

static void rb_podcast_source_show_browser 		(RBPodcastSource *source,
					   		 gboolean show);

static void rb_podcast_source_state_prefs_sync 		(RBPodcastSource *source);

static void album_selected_cb 				(RBPropertyView *propview, 
							 const char *name,
			 			       	 RBPodcastSource *podcast_source);

static void album_selection_reset_cb 			(RBPropertyView *propview, 
							 RBPodcastSource *podcast_source);

static void rb_podcast_source_songs_view_sort_order_changed_cb (RBEntryView *view, 
								RBPodcastSource *source);

static void rb_podcast_source_entry_activated_cb 	(RBEntryView *view, 
							 RhythmDBEntry *entry, 
							 RBPodcastSource *source);

static void rb_podcast_source_download_status_changed_cb(RBPodcastDownload *download, 
							 RhythmDBEntry *entry, 
							 gdouble status, 
							 RBPodcastSource *source);

static void rb_podcast_source_btn_file_change_cb 	(GtkFileChooserButton *widget, 
							 gpointer data);

static gboolean rb_podcast_source_spin_value_change_cb 	(GtkSpinButton *widget,
	       						 GdkEventFocus *event,	
							 gpointer data);

static gboolean rb_podcast_source_sync_config_cb 	(GtkWidget *widget, 
							 GdkEventFocus *event,
							 gpointer data);

void rb_podcast_source_show_columns_changed_cb 		(GtkToggleButton *button, 
							 RBPodcastSource *source);

/* source methods */
static char *impl_get_status 				(RBSource *source);
static const char *impl_get_browser_key 		(RBSource *source);
static GdkPixbuf *impl_get_pixbuf 			(RBSource *source);
static RBEntryView *impl_get_entry_view 		(RBSource *source);
static void impl_search 				(RBSource *source, const char *text);
static void impl_delete 				(RBSource *source);
static void impl_song_properties 			(RBSource *source);
static RBSourceEOFType impl_handle_eos 			(RBSource *asource);
static gboolean impl_show_popup 			(RBSource *source);
static void rb_podcast_source_do_query			(RBPodcastSource *source, RBPodcastQueryType type);
static GtkWidget *impl_get_config_widget 		(RBSource *source);


#define CMD_PATH_SHOW_BROWSER "/commands/ShowBrowser"
#define CMD_PATH_CURRENT_STATION "/commands/CurrentStation"
#define CMD_PATH_SONG_INFO    "/commands/SongInfo"

#define CONF_UI_PODCAST_DIR 			CONF_PREFIX "/ui/podcast"
#define CONF_UI_PODCAST_COLUMNS_SETUP 		CONF_PREFIX "/ui/podcast/columns_setup"
#define CONF_STATE_PODCAST_DIR 			CONF_PREFIX "/state/podcast"
#define CONF_STATE_PANED_POSITION 		CONF_PREFIX "/state/podcast/paned_position"
#define CONF_STATE_PODCAST_SORTING 		CONF_PREFIX "/state/podcast/sorting"
#define CONF_STATE_SHOW_BROWSER   		CONF_PREFIX "/state/podcast/show_browser"
#define CONF_STATE_SEARCH_TEXT   		CONF_PREFIX "/state/podcast/search_text"
#define CONF_STATE_PODCAST_DOWNLOAD_PREFIX	CONF_PREFIX "/state/podcast/download_prefix"
#define CONF_STATE_PODCAST_DOWNLOAD_HOUR	CONF_PREFIX "/state/podcast/download_hour"
#define CONF_STATE_PODCAST_DOWNLOAD_MINUTE	CONF_PREFIX "/state/podcast/download_minute"
#define CONF_STATE_PODCAST_DOWNLOAD_NEXT_TIME	CONF_PREFIX "/state/podcast/download_next_time"

struct RBPodcastSourcePrivate
{
	gboolean disposed;
	
	RhythmDB *db;

	GtkWidget *vbox;
	
	GtkWidget *config_widget;

	GdkPixbuf *pixbuf;

	RBPropertyView *albuns;
	RBEntryView *posts;

	char *title, *album; //, *name;
	double status;

	gboolean initialized;

	GtkWidget *paned;

	char *search_text;
	char *selected_album;

	RhythmDBEntryType entry_type;

	RhythmDBQueryModel *all_query;

	RBPodcastDownload *download_lst;	
};

struct RBDownloadData
{
	RhythmDBEntry *entry;
};

enum
{
	PROP_0,
	PROP_ENTRY_TYPE
};

static GObjectClass *parent_class = NULL;

GType
rb_podcast_source_get_type (void)
{
	static GType rb_podcast_source_type = 0;

	if (rb_podcast_source_type == 0)
	{
		static const GTypeInfo our_info =
		{
			sizeof (RBPodcastSourceClass),
			NULL,
			NULL,
			(GClassInitFunc) rb_podcast_source_class_init,
			NULL,
			NULL,
			sizeof (RBPodcastSource),
			0,
			(GInstanceInitFunc) rb_podcast_source_init
		};

		rb_podcast_source_type = g_type_register_static (RB_TYPE_SOURCE,
							      "RBPodcastSource",
							      &our_info, 0);
		
	}

	return rb_podcast_source_type;
}

static void
rb_podcast_source_class_init (RBPodcastSourceClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);
	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);

	object_class->dispose = rb_podcast_source_dispose;
	object_class->finalize = rb_podcast_source_finalize;
	object_class->constructor = rb_podcast_source_constructor;

	object_class->set_property = rb_podcast_source_set_property;
	object_class->get_property = rb_podcast_source_get_property;

	source_class->impl_get_status  = impl_get_status;
	source_class->impl_get_browser_key  = impl_get_browser_key;
	source_class->impl_get_pixbuf  = impl_get_pixbuf;
	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
	source_class->impl_search = impl_search;
	source_class->impl_get_config_widget = impl_get_config_widget;
	source_class->impl_search = impl_search;
	source_class->impl_get_entry_view = impl_get_entry_view;
	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_true_function;
	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
	source_class->impl_delete = impl_delete;
	source_class->impl_song_properties = impl_song_properties;
	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_true_function;
	source_class->impl_handle_eos = impl_handle_eos;
	source_class->impl_have_url = (RBSourceFeatureFunc) rb_true_function;
	source_class->impl_show_popup = impl_show_popup;

	g_object_class_install_property (object_class,
					 PROP_ENTRY_TYPE,
					 g_param_spec_uint ("entry-type",
							    "Entry type",
							    "Type of the entries which should be displayed by this source",
							    0,
							    G_MAXINT,
							    RHYTHMDB_ENTRY_TYPE_PODCAST_POST,
							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	
}

static void
rb_podcast_source_init (RBPodcastSource *source)
{
	GtkWidget *dummy = gtk_tree_view_new ();
	source->priv = g_new0 (RBPodcastSourcePrivate, 1);

	source->priv->vbox = gtk_vbox_new (FALSE, 5);

	gtk_container_add (GTK_CONTAINER (source), source->priv->vbox);
	
	source->priv->title = g_strdup (_("Unknown"));
	source->priv->album = g_strdup (_("Unknown"));
	source->priv->status = 0;


	source->priv->pixbuf = gtk_widget_render_icon (dummy,
						       RB_STOCK_PODCAST,
						       GTK_ICON_SIZE_LARGE_TOOLBAR,
						       NULL);

	
	gtk_widget_destroy (dummy);
}

static void
rb_podcast_source_dispose (GObject *object)
{
	RBPodcastSource *source;

	rb_debug("dispose podcast_source");
	source = RB_PODCAST_SOURCE (object);
	if (source->priv->disposed)
		return;
	
	source->priv->disposed = TRUE;

	g_object_unref (source->priv->download_lst);
	g_object_unref (source->priv->all_query);
}

static void
rb_podcast_source_finalize (GObject *object)
{
	RBPodcastSource *source;

	g_return_if_fail (object != NULL);
	g_return_if_fail (RB_IS_PODCAST_SOURCE (object));

	source = RB_PODCAST_SOURCE (object);

	g_return_if_fail (source->priv != NULL);

	rb_debug ("finalizing podcast source");

	g_free (source->priv->title);
	g_free (source->priv->album);

	g_free (source->priv);


	G_OBJECT_CLASS (parent_class)->finalize (object);
}

static GObject *
rb_podcast_source_constructor (GType type, guint n_construct_properties,
			      GObjectConstructParam *construct_properties)
{
	RBPodcastSource *source;
	RBPodcastSourceClass *klass;
	GObjectClass *parent_class;  
	RBShell *shell;

	klass = RB_PODCAST_SOURCE_CLASS (g_type_class_peek (RB_TYPE_PODCAST_SOURCE));

	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	source = RB_PODCAST_SOURCE (parent_class->constructor (type, n_construct_properties,
							      construct_properties));

	source->priv->paned = gtk_hpaned_new ();

	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
	g_object_get (G_OBJECT (shell), "db", &source->priv->db, NULL);
	g_object_unref (G_OBJECT (shell));


	source->priv->download_lst = rb_podcast_download_new(source->priv->db);
	rb_podcast_download_start_sync (source->priv->download_lst);

	/* set up posts view */
	source->priv->posts = rb_entry_view_new (source->priv->db, CONF_STATE_PODCAST_SORTING,
						    FALSE, FALSE);

	rb_entry_view_append_column (source->priv->posts, RB_ENTRY_VIEW_COL_DATE_STR);
	rb_entry_view_append_column (source->priv->posts, RB_ENTRY_VIEW_COL_TITLE);
	rb_entry_view_append_column (source->priv->posts, RB_ENTRY_VIEW_COL_ALBUM);
	rb_entry_view_append_column (source->priv->posts, RB_ENTRY_VIEW_COL_STATUS);
	g_signal_connect_object (G_OBJECT (source->priv->posts),
				 "sort-order-changed",
				 G_CALLBACK (rb_podcast_source_songs_view_sort_order_changed_cb),
				 source, 0);

	g_signal_connect (G_OBJECT (source->priv->download_lst),
			"status_changed",
			G_CALLBACK (rb_podcast_source_download_status_changed_cb),
			source);


		   
	g_signal_connect_object (G_OBJECT (source->priv->posts),
					"entry-activated",
					G_CALLBACK (rb_podcast_source_entry_activated_cb),
				        source, 0);

	g_signal_connect_object (G_OBJECT (source->priv->posts),
				 "size_allocate",
				 G_CALLBACK (paned_size_allocate_cb),
				 source, 0);
	
	g_signal_connect_object (G_OBJECT (source->priv->posts), "show_popup",
				 G_CALLBACK (rb_podcast_source_songs_show_popup_cb), source, 0);
	
	/* set up genre entry view */
	source->priv->albuns = rb_property_view_new (source->priv->db,
						     RHYTHMDB_PROP_ALBUM,
						     _("Site"));


	g_signal_connect_object (G_OBJECT (source->priv->albuns),
				 "property-selected",
				 G_CALLBACK (album_selected_cb),
				 source, 0);

	g_signal_connect_object (G_OBJECT (source->priv->albuns),
				 "property-selection-reset",
				 G_CALLBACK (album_selection_reset_cb),
				 source, 0);

	g_object_set (G_OBJECT (source->priv->albuns), "vscrollbar_policy",
		      GTK_POLICY_AUTOMATIC, NULL);


	g_object_ref (G_OBJECT (source->priv->albuns));


	gtk_paned_pack2 (GTK_PANED (source->priv->paned),
			 GTK_WIDGET (source->priv->posts), TRUE, FALSE);

	gtk_box_pack_start_defaults (GTK_BOX (source->priv->vbox), source->priv->paned);

	

	rb_podcast_source_state_prefs_sync (source);
	
	
	eel_gconf_notification_add (CONF_STATE_PODCAST_DIR,
				    (GConfClientNotifyFunc) rb_podcast_source_state_pref_changed,
				    source);
	
	gtk_widget_show_all (GTK_WIDGET (source));

	rb_podcast_source_do_query (source, RB_PODCAST_QUERY_TYPE_ALL);

	return G_OBJECT (source);
}

static void
rb_podcast_source_set_property (GObject *object,
			      guint prop_id,
			      const GValue *value,
			      GParamSpec *pspec)
{
	RBPodcastSource *source = RB_PODCAST_SOURCE (object);

	switch (prop_id)
	{
	case PROP_ENTRY_TYPE:
		source->priv->entry_type = g_value_get_uint (value);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

static void
rb_podcast_source_get_property (GObject *object,
			      guint prop_id,
			      GValue *value,
			      GParamSpec *pspec)
{
	RBPodcastSource *source = RB_PODCAST_SOURCE (object);

	switch (prop_id)
	{
	case PROP_ENTRY_TYPE:
		g_value_set_uint (value, source->priv->entry_type);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

RBSource *
rb_podcast_source_new (RBShell *shell)
{
	RBSource *source;
	source = RB_SOURCE (g_object_new (RB_TYPE_PODCAST_SOURCE,
					  "name", _("Podcast"),
					  "shell", shell,
					  NULL));

	rb_shell_register_entry_type_for_source (shell, source, 
						 RHYTHMDB_ENTRY_TYPE_PODCAST_POST);


	

	return source;
}

static GdkPixbuf *
impl_get_pixbuf (RBSource *asource)
{
	RBPodcastSource *source = RB_PODCAST_SOURCE (asource);

	return source->priv->pixbuf;
}

static void
impl_search (RBSource *asource, const char *search_text)
{
	RBPodcastSource *source = RB_PODCAST_SOURCE (asource);

	if (source->priv->initialized) {
		if (search_text == NULL && source->priv->search_text == NULL)
			return;
		if (search_text != NULL &&
		    source->priv->search_text != NULL
		    && !strcmp (search_text, source->priv->search_text))
			return;
	}

	source->priv->initialized = TRUE;
	if (search_text != NULL && search_text[0] == '\0')
		search_text = NULL;

	g_free (source->priv->search_text);
	if (search_text)
		source->priv->search_text = g_utf8_casefold (search_text, -1);
	else
		source->priv->search_text = NULL;
	rb_podcast_source_do_query (source, RB_PODCAST_QUERY_TYPE_SEARCH);

	rb_source_notify_filter_changed (RB_SOURCE (source));
}

static RBEntryView *
impl_get_entry_view (RBSource *asource)
{
	RBPodcastSource *source = RB_PODCAST_SOURCE (asource);

	return source->priv->posts;
}

static RBSourceEOFType
impl_handle_eos (RBSource *asource)
{
	return RB_SOURCE_EOF_ERROR;
}

static char *
impl_get_status (RBSource *asource)
{
	
 	RBPodcastSource *source = RB_PODCAST_SOURCE (asource);
	char *ret;
	guint num_entries = rb_entry_view_get_num_entries (source->priv->posts);
	ret = g_strdup_printf (ngettext ("%d posts", "%d posts", num_entries),
			       num_entries);
			       
	return ret;
}

static const char *
impl_get_browser_key (RBSource *asource)
{
	return CONF_STATE_SHOW_BROWSER;
}

static void
impl_delete (RBSource *asource)
{
	RBPodcastSource *source = RB_PODCAST_SOURCE (asource);
	GList *l;

	for (l = rb_entry_view_get_selected_entries (source->priv->posts); l != NULL;
	     l = g_list_next (l)) {
		rhythmdb_entry_delete (source->priv->db, l->data);
		rhythmdb_commit (source->priv->db);
	}
}

static void
impl_song_properties (RBSource *asource)
{
	RBPodcastSource *source = RB_PODCAST_SOURCE (asource);
	GtkWidget *dialog = rb_podcast_properties_dialog_new (source->priv->posts);
	rb_debug ("in song properties");
	if (dialog)
		gtk_widget_show_all (dialog);
	else
		rb_debug ("no selection!");
}

static void
paned_size_allocate_cb (GtkWidget *widget,
			GtkAllocation *allocation,
		        RBPodcastSource *source)
{
	rb_debug ("paned size allocate");
	eel_gconf_set_integer (CONF_STATE_PANED_POSITION,
			       gtk_paned_get_position (GTK_PANED (source->priv->paned)));
}

static void
rb_podcast_source_state_prefs_sync (RBPodcastSource *source)
{
	rb_debug ("syncing state");
	gtk_paned_set_position (GTK_PANED (source->priv->paned),
				eel_gconf_get_integer (CONF_STATE_PANED_POSITION));
	rb_podcast_source_show_browser (source,
				       eel_gconf_get_boolean (CONF_STATE_SHOW_BROWSER));
}

static void
rb_podcast_source_state_pref_changed (GConfClient *client,
				     guint cnxn_id,
				     GConfEntry *entry,
				     RBPodcastSource *source)
{
	rb_debug ("state prefs changed");
	rb_podcast_source_state_prefs_sync (source);
}

static void
rb_podcast_source_songs_view_sort_order_changed_cb (RBEntryView *view,
						   RBPodcastSource *source)
{
	rb_debug ("sort order changed");
	rb_entry_view_set_resorting (view);
	rb_podcast_source_do_query (source, RB_PODCAST_QUERY_TYPE_SEARCH);
}

static void
rb_podcast_source_download_status_changed_cb (RBPodcastDownload *download,
						RhythmDBEntry *entry,
						gdouble status,
						RBPodcastSource *source)
{
	gtk_widget_queue_draw(GTK_WIDGET(source->priv->posts));
	return;

}


static void
rb_podcast_source_songs_show_popup_cb (RBEntryView *view,
				      RBPodcastSource *source)
{
	if (G_OBJECT (source) == NULL) {
		return;
	}
	_rb_source_show_popup (RB_SOURCE (source), "/PodcastViewPopup");
}


static void
album_selected_cb (RBPropertyView *propview, const char *name,
		   RBPodcastSource *podcast_source)
{
	g_free (podcast_source->priv->selected_album);
	podcast_source->priv->selected_album = g_strdup (name);
	rb_podcast_source_do_query (podcast_source, RB_PODCAST_QUERY_TYPE_ALBUM);

	rb_source_notify_filter_changed (RB_SOURCE (podcast_source));
	
}

static void
album_selection_reset_cb (RBPropertyView *propview, RBPodcastSource *podcast_source)
{
	g_free (podcast_source->priv->selected_album);
	podcast_source->priv->selected_album = NULL;

	rb_podcast_source_do_query (podcast_source, RB_PODCAST_QUERY_TYPE_ALBUM);

	rb_source_notify_filter_changed (RB_SOURCE (podcast_source));
	
}

static void
rb_podcast_source_show_browser (RBPodcastSource *source,
			       gboolean show)
{
	
	GtkWidget *albunswidget = GTK_WIDGET (source->priv->albuns);

	if (show == TRUE) {
		gtk_paned_pack1 (GTK_PANED (source->priv->paned), albunswidget, FALSE, FALSE);
		gtk_widget_show_all (albunswidget);
	} else if (show == FALSE) {
		GList *children = gtk_container_get_children (GTK_CONTAINER (source->priv->paned));
		gtk_widget_hide (albunswidget);
		if (g_list_find (children, albunswidget))
		    gtk_container_remove (GTK_CONTAINER (source->priv->paned), albunswidget);
		g_list_free (children);
	}
}

static void
rb_podcast_source_do_query (RBPodcastSource *source, RBPodcastQueryType qtype)
{
	RhythmDBEntryType entry_type;
	RhythmDBQueryModel *query_model;
	RhythmDBPropertyModel *genre_model;
	GtkTreeModel *model;
	GPtrArray *query;

	if ((qtype == RB_PODCAST_QUERY_TYPE_ALL) && (source->priv->all_query)) {
		rb_debug ("using cached query");
		rb_entry_view_set_model (source->priv->posts, RHYTHMDB_QUERY_MODEL (source->priv->all_query));
		return;
	}

	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
	query = rhythmdb_query_parse (source->priv->db,
				      RHYTHMDB_QUERY_PROP_EQUALS,
				      RHYTHMDB_PROP_TYPE,
				      entry_type,
				      RHYTHMDB_QUERY_END);
	query_model = rhythmdb_query_model_new_empty (source->priv->db);

	if (qtype == RB_PODCAST_QUERY_TYPE_ALL) {
		rb_debug ("building new all query");
		rb_property_view_reset (source->priv->albuns);
		g_free (source->priv->selected_album);
		source->priv->selected_album = NULL;

		genre_model = rb_property_view_get_model (source->priv->albuns);
		g_object_set (G_OBJECT (genre_model), "query-model", query_model, NULL);

		source->priv->all_query = query_model;
	} else {
		if (source->priv->search_text) {
			GPtrArray *subquery = rhythmdb_query_parse (source->priv->db,
								    RHYTHMDB_QUERY_PROP_LIKE,
								    RHYTHMDB_PROP_ALBUM_FOLDED,
								    source->priv->search_text,
								    RHYTHMDB_QUERY_DISJUNCTION,
								    RHYTHMDB_QUERY_PROP_LIKE,
								    RHYTHMDB_PROP_TITLE_FOLDED,
								    source->priv->search_text,
								    RHYTHMDB_QUERY_END);
			rhythmdb_query_append (source->priv->db,
					       query,
					       RHYTHMDB_QUERY_SUBQUERY,
					       subquery,
					       RHYTHMDB_QUERY_END);
		}


		if (source->priv->selected_album)
			rhythmdb_query_append (source->priv->db,
					       query,
					       RHYTHMDB_QUERY_PROP_EQUALS,
					       RHYTHMDB_PROP_ALBUM,
					       source->priv->selected_album,
					       RHYTHMDB_QUERY_END);
	}


	model = GTK_TREE_MODEL (query_model);
	
	rb_entry_view_set_model (source->priv->posts, RHYTHMDB_QUERY_MODEL (query_model));
	if (query_model != source->priv->all_query)
		rhythmdb_query_model_set_connected (source->priv->all_query, TRUE);

	rhythmdb_do_full_query_parsed (source->priv->db, model, query);

	rhythmdb_query_free (query);

}

static gboolean
impl_show_popup (RBSource *source)
{
	return TRUE;
}


static void 
rb_podcast_source_entry_activated_cb (RBEntryView *view, RhythmDBEntry *entry, RBPodcastSource *source)
{
}

static GtkWidget *
impl_get_config_widget (RBSource *asource)
{
	RBPodcastSource *source = RB_PODCAST_SOURCE (asource);
	GtkWidget *spin_hour;
	GtkWidget *spin_minute;
	GtkWidget *btn_file;
	GtkWidget *v_box;
	const char *download_dir;
	GladeXML *xml;

	if (source->priv->config_widget)
		return source->priv->config_widget;


	xml = rb_glade_xml_new ("podcast-prefs.glade", "podcast_vbox", source);
	source->priv->config_widget =
		glade_xml_get_widget (xml, "podcast_vbox");
	
	v_box = glade_xml_get_widget (xml, "vbox11");
	

	btn_file = gtk_file_chooser_button_new(_("Save Folder"), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
	gtk_box_pack_start (GTK_BOX(v_box), btn_file, FALSE, TRUE, 0); 
	
	download_dir = 	eel_gconf_get_string(CONF_STATE_PODCAST_DOWNLOAD_PREFIX);

	if (download_dir == NULL) {
		download_dir = g_build_filename (g_get_home_dir(),
							"My Podcast",
							NULL);
	}
		

	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER(btn_file), 
						download_dir);

	g_signal_connect ( G_OBJECT(btn_file),
				"selection-changed",
				G_CALLBACK (rb_podcast_source_btn_file_change_cb),
				CONF_STATE_PODCAST_DOWNLOAD_PREFIX);

	
	spin_hour = glade_xml_get_widget (xml, "spin_hour");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_hour), eel_gconf_get_integer (CONF_STATE_PODCAST_DOWNLOAD_HOUR));
	g_signal_connect ( G_OBJECT(spin_hour),
				"focus-out-event",
				G_CALLBACK(rb_podcast_source_spin_value_change_cb),
				CONF_STATE_PODCAST_DOWNLOAD_HOUR);
				
	spin_minute = glade_xml_get_widget (xml, "spin_minute");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_minute), eel_gconf_get_integer (CONF_STATE_PODCAST_DOWNLOAD_MINUTE));
	g_signal_connect ( G_OBJECT(spin_minute),
				"focus-out-event",
				G_CALLBACK(rb_podcast_source_spin_value_change_cb),
				CONF_STATE_PODCAST_DOWNLOAD_MINUTE);
	
	g_signal_connect ( G_OBJECT(spin_minute),
				"focus-out-event",
				G_CALLBACK(rb_podcast_source_sync_config_cb),
				source);

	g_signal_connect ( G_OBJECT(spin_hour),
				"focus-out-event",
				G_CALLBACK(rb_podcast_source_sync_config_cb),
				source);

	
	g_object_unref (G_OBJECT (xml));
	
	return source->priv->config_widget;
}

static void 
rb_podcast_source_btn_file_change_cb (GtkFileChooserButton *widget, gpointer data)
{

	eel_gconf_set_string ((const char *) data, 
				gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(widget)));
}

static gboolean
rb_podcast_source_spin_value_change_cb (GtkSpinButton *widget, GdkEventFocus *event, gpointer data)
{
	eel_gconf_set_integer ((const char *) data,
				(guint ) gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget)));

	return FALSE;

}

static gboolean
rb_podcast_source_sync_config_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
{
	RBPodcastSource *source = (RBPodcastSource *) data;

	guint hour = eel_gconf_get_integer (CONF_STATE_PODCAST_DOWNLOAD_HOUR);
	guint minute = eel_gconf_get_integer (CONF_STATE_PODCAST_DOWNLOAD_MINUTE);

	hour = hour * 3600;
	minute = minute * 60;

	eel_gconf_set_integer (CONF_STATE_PODCAST_DOWNLOAD_NEXT_TIME, hour + minute);

	rb_podcast_download_start_sync( source->priv->download_lst );
	
	return FALSE;
}


